package com.citygis.service;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FSDataOutputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IOUtils;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.sql.*;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Map;

public class HadoopTest {
    // 可操作HDFS文件系统的对象
    FileSystem hdfs = null;

    @Before
    public void init() throws IOException {
        // 构造一个配置参数对象，设置一个参数：要访问的HDFS的URI
        Configuration conf = new Configuration();
        // 指定使用HDFS访问
        conf.set("fs.defaultFS", "hdfs://hdcluster");
        // 进行客户端身份的设置(root为虚拟机的用户名，hadoop集群节点的其中一个都可以)
        System.setProperty("HADOOP_USER_NAME", "bdp");
        // 通过FileSystem的静态get()方法获取HDFS文件系统客户端对象
        hdfs = FileSystem.get(conf);
    }

    @After
    public void close() throws IOException {
        // 关闭文件操作对象
        hdfs.close();
    }

    @Test
    public void testUploadFileToHDFS() throws IOException {
        // 待上传的文件路径(windows)
        Path src = new Path("C:\\Users\\csdl\\Desktop\\test.xlsx");
        // 上传之后存放的路径(HDFS)
        Path dst = new Path("/test/test.xlsx");
        // 上传
        hdfs.copyFromLocalFile(src, dst);
        System.out.println("上传成功");
    }

    @Test
    public void testDownFileToLocal() throws IOException {
        // HDFS文件路径和本地文件路径
        String hdfsFilePath = "/test/test.xlsx";
        String localFilePath = "D:\\work\\工作文件\\疾控\\fileTest/test1.xlsx";
//        // 待下载的路径(HDFS)
//        Path src = new Path("/HDFSTest.txt");
//        // 下载成功之后存放的路径(windows)
//        Path dst = new Path("D:\\work\\工作文件\\疾控\\fileTest/HDFSTest1.txt");
//        // 下载
//        hdfs.copyToLocalFile(false,src,dst,true);
        // 获取数据流
        FSDataInputStream open = hdfs.open(new Path(hdfsFilePath));
        OutputStream output = new FileOutputStream(localFilePath);
        IOUtils.copyBytes(open, output, 4096, true);
        System.out.println("下载成功");
    }


    @Test
    public void testSqlServer100() throws IOException {
        try {
            System.out.println("开始时间" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));
            Path path = new Path("/chengdi/ddl_test_sqlserver_100/ddl_test_sqlserver_100.txt"); // HDFS上的文件路径

            // 如果文件已存在，则覆盖
            FSDataOutputStream outputStream = hdfs.create(path);


            Class.forName("com.microsoft.sqlserver.jdbc.SQLServerDriver");
            String jdbcUrl2 = "jdbc:sqlserver://localhost:1433;DatabaseName=master";
            Connection conn2 = DriverManager.getConnection(jdbcUrl2, "sa", "123456");

            PreparedStatement ps2 = conn2.prepareStatement("SELECT count(1) FROM [dbo].[TABLE_100w]");
            ResultSet rs = ps2.executeQuery();

            int total = 0;
            while (rs.next()) {
                total = (int) rs.getLong(1);
            }
            System.out.println("总数据量：" + total);
            ps2.close();

            Map<String, Object> map = new HashMap<>();
            int pageIndex = 1;
            int pageSize = 200000;
            int begin = pageSize * (pageIndex - 1);
            map.put("begin", begin);
            map.put("pageSize", pageSize);

            String data = "";
            while (begin < total) {
                System.out.println(begin);
                map.put("begin", begin);
                PreparedStatement ps_sql = conn2.prepareStatement("select * from [dbo].[TABLE_100w] order by COLUMN_2 OFFSET ? ROWS FETCH NEXT ? ROWS ONLY;");
                ps_sql.setInt(1, begin);
                ps_sql.setInt(2, pageSize);
                ResultSet rs2 = ps_sql.executeQuery();

                if(begin>0){
                    outputStream = hdfs.append(path);
                }

                while (rs2.next()){
                    // 写入数据
                    data = rs2.getString(1)+","
                            +rs2.getInt(2)+","
                            +rs2.getString(3)+","
                            +rs2.getBigDecimal(4)+","
                            +rs2.getDate(5)+","
                            +rs2.getString(6)+","
                            +rs2.getString(7)+","
                            +rs2.getString(8)+","
                            +rs2.getString(9)+","
                            +rs2.getString(10)+","
                            +rs2.getString(11)+","
                            +rs2.getString(12)+","
                            + "\n";
                    byte[] bytes = data.getBytes();
                    outputStream.write(bytes);
                }
                pageIndex++;
                begin = pageSize * (pageIndex - 1);
                // 关闭流
                outputStream.close();
            }
            conn2.close();




            hdfs.close();

            System.out.println("Data written to file successfully.");
            System.out.println("结束时间" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));

        } catch (IOException | ClassNotFoundException | SQLException e) {
            e.printStackTrace();
        }
    }

    @Test
    public void testSqlServer500() throws IOException {
        try {
            System.out.println("开始时间" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));
            Path path = new Path("/chengdi/ddl_test_sqlserver_500/ddl_test_sqlserver_500.txt"); // HDFS上的文件路径

            // 如果文件已存在，则覆盖
            FSDataOutputStream outputStream = hdfs.create(path);


            Class.forName("com.microsoft.sqlserver.jdbc.SQLServerDriver");
            String jdbcUrl2 = "jdbc:sqlserver://localhost:1433;DatabaseName=master";
            Connection conn2 = DriverManager.getConnection(jdbcUrl2, "sa", "123456");

            PreparedStatement ps2 = conn2.prepareStatement("SELECT count(1) FROM [dbo].[TABLE_500w]");
            ResultSet rs = ps2.executeQuery();

            int total = 0;
            while (rs.next()) {
                total = (int) rs.getLong(1);
            }
            System.out.println("总数据量：" + total);
            ps2.close();

            Map<String, Object> map = new HashMap<>();
            int pageIndex = 1;
            int pageSize = 200000;
            int begin = pageSize * (pageIndex - 1);
            map.put("begin", begin);
            map.put("pageSize", pageSize);

            String data = "";
            while (begin < total) {
                System.out.println(begin);
                map.put("begin", begin);
                PreparedStatement ps_sql = conn2.prepareStatement("select * from [dbo].[TABLE_500w] order by COLUMN_2 OFFSET ? ROWS FETCH NEXT ? ROWS ONLY;");
                ps_sql.setInt(1, begin);
                ps_sql.setInt(2, pageSize);
                ResultSet rs2 = ps_sql.executeQuery();

                if(begin>0){
                    outputStream = hdfs.append(path);
                }

                while (rs2.next()){
                    // 写入数据
                    data = rs2.getString(1)+","
                            +rs2.getInt(2)+","
                            +rs2.getString(3)+","
                            +rs2.getBigDecimal(4)+","
                            +rs2.getDate(5)+","
                            +rs2.getString(6)+","
                            +rs2.getString(7)+","
                            +rs2.getString(8)+","
                            +rs2.getString(9)+","
                            +rs2.getString(10)+","
                            +rs2.getString(11)+","
                            +rs2.getString(12)+","
                            + "\n";
                    byte[] bytes = data.getBytes();
                    outputStream.write(bytes);
                }
                pageIndex++;
                begin = pageSize * (pageIndex - 1);
                // 关闭流
                outputStream.close();
            }
            conn2.close();




            hdfs.close();

            System.out.println("Data written to file successfully.");
            System.out.println("结束时间" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));

        } catch (IOException | ClassNotFoundException | SQLException e) {
            e.printStackTrace();
        }
    }
}
