package com.citygis.service;

import org.apache.kafka.clients.producer.*;
import org.apache.kafka.clients.consumer.*;
import org.junit.Test;
import sun.text.resources.FormatData;

import javax.xml.crypto.Data;
import java.sql.*;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;
import java.util.Properties;

class KafkaTest {
    private static final String TOPIC = "test_topic";
    private static final String BOOTSTRAP_SERVERS = "localhost:9092";

    public static void main(String[] args) throws SQLException, ClassNotFoundException {

        producer();
//        testQuery500w();
//        testQueryfei100w();
//        testQueryfei500w();

    }

    static void producer() {
        // 生产者示例
        Properties producerProps = new Properties();
        producerProps.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, BOOTSTRAP_SERVERS);
        producerProps.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringSerializer");
        producerProps.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringSerializer");

        Producer<String, String> producer = new KafkaProducer<>(producerProps);

        String message = "{\"id\":\"1005\",\"line\":\"张三\",\"line2\":\"手机\",\"line3\":\"上海\"}";
        ProducerRecord<String, String> records = new ProducerRecord<>(TOPIC, message);

        System.out.println("开始时间" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));

        for (int i = 1; i <= 1000000; i++) {
            int finalI = i;
            producer.send(records, new Callback() {
                @Override
                public void onCompletion(RecordMetadata metadata, Exception exception) {
                    if (exception != null) {
                        System.err.println("Failed to send message: " + exception.getMessage());
                    } else {
                        System.out.println("Sent message: topic=" + TOPIC + ", offset=%d%n,%n" + finalI);
                    }
                }
            });
        }
        producer.close();
        System.out.println("结束时间" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));
    }


    static void consumer() {
        // 消费者示例
        Properties consumerProps = new Properties();
        consumerProps.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, BOOTSTRAP_SERVERS);
        consumerProps.put(ConsumerConfig.GROUP_ID_CONFIG, "test-group");
        consumerProps.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringDeserializer");
        consumerProps.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringDeserializer");
        Consumer<String, String> consumer = new KafkaConsumer<>(consumerProps);
        consumer.subscribe(Collections.singletonList(TOPIC));
        while (true) {
            ConsumerRecords<String, String> recordss = consumer.poll(100);
            for (ConsumerRecord<String, String> record : recordss) {
                System.out.printf("Received message: topic=%s, partition=%d, offset=%d, key=%s, value=%s%n",
                        record.topic(), record.partition(), record.offset(), record.key(), record.value());
            }
        }
    }

//            test_result_json_100w  非结构
//            test_result_json_500w

//            test_result_500w  结构
//            test_result

    public static void testQuery100w() throws SQLException, ClassNotFoundException {
        System.out.println("开始时间" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));
        Class.forName("org.apache.hive.jdbc.HiveDriver");
        String jdbcUrl = "jdbc:hive2://10.1.161.88:10000/default";
        Connection conn = DriverManager.getConnection(jdbcUrl);

        PreparedStatement ps = conn.prepareStatement("SELECT count(1) FROM test_result_json_100w");
        ResultSet rs = ps.executeQuery();

        int total = 0;
        while (rs.next()) {
            total = (int) rs.getLong(1);
        }
        System.out.println("总数据量：" + total);
        ps.close();

        int rowCount = 0;
//        Map<String, Object> map = new HashMap<>();
        int pageIndex = 1;
        int pageSize = 1000000;
        int begin = pageSize * (pageIndex - 1);
//        map.put("begin",begin);
//        map.put("pageSize",pageSize);

        System.out.println("开始查询" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));
        while (begin < total) {
//        while (begin <= total) {
//            System.out.println(begin);
            PreparedStatement ps2 = conn.prepareStatement("SELECT * FROM test_result_json_100w LIMIT ?,?");
            ps2.setInt(1, begin);
            ps2.setInt(2, pageSize);

            ResultSet rs2 = ps2.executeQuery();
            ResultSetMetaData rsmd = rs2.getMetaData();
            int size = rsmd.getColumnCount();
            while (rs2.next()) {
//                StringBuffer value = new StringBuffer();
//                for (int i = 0; i < size; i++) {
//                    value.append(rs2.getString(i + 1)).append("\t");
//                }
//                // 获取输出结果并关闭连接
//                System.out.println(value.toString());
                rowCount++;
            }

            System.out.println(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()) + "  查询总行数" + rowCount);

            rs2.close();
            ps2.close();

            pageIndex++;
            begin = pageSize * (pageIndex - 1);
        }
        conn.close();
        System.out.println("结束时间" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));

    }

    public static void testQuery500w() throws SQLException, ClassNotFoundException {
        System.out.println("开始时间" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));
        Class.forName("org.apache.hive.jdbc.HiveDriver");
        String jdbcUrl = "jdbc:hive2://10.1.161.88:10000/default";
        Connection conn = DriverManager.getConnection(jdbcUrl);

        PreparedStatement ps = conn.prepareStatement("SELECT count(1) FROM test_result_json_500w");
        ResultSet rs = ps.executeQuery();

        int total = 0;
        while (rs.next()) {
            total = (int) rs.getLong(1);
        }
        System.out.println("总数据量：" + total);
        ps.close();

        int rowCount = 0;
//        Map<String, Object> map = new HashMap<>();
        int pageIndex = 1;
        int pageSize = 1000000;
        int begin = pageSize * (pageIndex - 1);
//        map.put("begin",begin);
//        map.put("pageSize",pageSize);

        System.out.println("开始查询" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));
        while (begin < total) {
//        while (begin <= total) {
//            System.out.println(begin);
            PreparedStatement ps2 = conn.prepareStatement("SELECT * FROM test_result_json_500w LIMIT ?,?");
            ps2.setInt(1, begin);
            ps2.setInt(2, pageSize);

            ResultSet rs2 = ps2.executeQuery();
            ResultSetMetaData rsmd = rs2.getMetaData();
            int size = rsmd.getColumnCount();
            while (rs2.next()) {
//                StringBuffer value = new StringBuffer();
//                for (int i = 0; i < size; i++) {
//                    value.append(rs2.getString(i + 1)).append("\t");
//                }
//                // 获取输出结果并关闭连接
//                System.out.println(value.toString());
                rowCount++;
            }

            System.out.println(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()) + "  查询总行数" + rowCount);

            rs2.close();
            ps2.close();

            pageIndex++;
            begin = pageSize * (pageIndex - 1);
        }
        conn.close();
        System.out.println("结束时间" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));

    }

    public static void testQueryfei500w() throws SQLException, ClassNotFoundException {
        System.out.println("开始时间" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));
        Class.forName("org.apache.hive.jdbc.HiveDriver");
        String jdbcUrl = "jdbc:hive2://10.1.161.88:10000/default";
        Connection conn = DriverManager.getConnection(jdbcUrl);

        PreparedStatement ps = conn.prepareStatement("SELECT count(1) FROM test_result_500w");
        ResultSet rs = ps.executeQuery();

        int total = 0;
        while (rs.next()) {
            total = (int) rs.getLong(1);
        }
        System.out.println("总数据量：" + total);
        ps.close();

        int rowCount = 0;
//        Map<String, Object> map = new HashMap<>();
        int pageIndex = 1;
        int pageSize = 1000000;
        int begin = pageSize * (pageIndex - 1);
//        map.put("begin",begin);
//        map.put("pageSize",pageSize);

        System.out.println("开始查询" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));
        while (begin < total) {
//        while (begin <= total) {
//            System.out.println(begin);
            PreparedStatement ps2 = conn.prepareStatement("SELECT * FROM test_result_500w LIMIT ?,?");
            ps2.setInt(1, begin);
            ps2.setInt(2, pageSize);

            ResultSet rs2 = ps2.executeQuery();
            ResultSetMetaData rsmd = rs2.getMetaData();
            int size = rsmd.getColumnCount();
            while (rs2.next()) {
//                StringBuffer value = new StringBuffer();
//                for (int i = 0; i < size; i++) {
//                    value.append(rs2.getString(i + 1)).append("\t");
//                }
//                // 获取输出结果并关闭连接
//                System.out.println(value.toString());
                rowCount++;
            }

            System.out.println(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()) + "  查询总行数" + rowCount);

            rs2.close();
            ps2.close();

            pageIndex++;
            begin = pageSize * (pageIndex - 1);
        }
        conn.close();
        System.out.println("结束时间" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));

    }

    public static void testQueryfei100w() throws SQLException, ClassNotFoundException {
        System.out.println("开始时间" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));
        Class.forName("org.apache.hive.jdbc.HiveDriver");
        String jdbcUrl = "jdbc:hive2://10.1.161.88:10000/default";
        Connection conn = DriverManager.getConnection(jdbcUrl);

        PreparedStatement ps = conn.prepareStatement("SELECT count(1) FROM test_result");
        ResultSet rs = ps.executeQuery();

        int total = 0;
        while (rs.next()) {
            total = (int) rs.getLong(1);
        }
        System.out.println("总数据量：" + total);
        ps.close();

        int rowCount = 0;
//        Map<String, Object> map = new HashMap<>();
        int pageIndex = 1;
        int pageSize = 1000000;
        int begin = pageSize * (pageIndex - 1);
//        map.put("begin",begin);
//        map.put("pageSize",pageSize);

        System.out.println("开始查询" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));
        while (begin < total) {
//        while (begin <= total) {
//            System.out.println(begin);
            PreparedStatement ps2 = conn.prepareStatement("SELECT * FROM test_result LIMIT ?,?");
            ps2.setInt(1, begin);
            ps2.setInt(2, pageSize);

            ResultSet rs2 = ps2.executeQuery();
            ResultSetMetaData rsmd = rs2.getMetaData();
            int size = rsmd.getColumnCount();
            while (rs2.next()) {
//                StringBuffer value = new StringBuffer();
//                for (int i = 0; i < size; i++) {
//                    value.append(rs2.getString(i + 1)).append("\t");
//                }
//                // 获取输出结果并关闭连接
//                System.out.println(value.toString());
                rowCount++;
            }

            System.out.println(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()) + "  查询总行数" + rowCount);

            rs2.close();
            ps2.close();

            pageIndex++;
            begin = pageSize * (pageIndex - 1);
        }
        conn.close();
        System.out.println("结束时间" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss SSS").format(new java.util.Date()));

    }
}
