package com.citygis.service;

import org.junit.Test;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.Random;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

public class generateFile {

    @Test
    public void generateJson() {
        String jsonFilePath = "E:\\非结构化数据\\output_bz.json";
//        String jsonFilePath = "D:\\Work\\CSDL\\非结构化数据\\output_bz.json";
        long fileSizeLimit = 1_073_741_824 * 100L;
//        long fileSizeLimit = 1_073_741_824;
        try (BufferedWriter writer = Files.newBufferedWriter(Paths.get(jsonFilePath))) {
            writer.write("[\n");
            File file = new File(jsonFilePath);
            long currentSize = file.length();

            while (currentSize < fileSizeLimit) {
                // 将对象转换为JSON字符串
                String jsonData = generateJsonData();

                // 写入文件之前可以检查是否这次写入会导致文件超过限制
                if (currentSize + jsonData.getBytes(StandardCharsets.UTF_8).length > fileSizeLimit) {
                    System.out.println("Reached file size limit. Stopping.");
                    writer.write("]\n");
                    break;
                }

                // 写入JSON数据到文件
                writer.write(jsonData);
                writer.newLine(); // 可选，根据需要添加换行符以便区分不同的JSON对象

                currentSize += jsonData.getBytes(StandardCharsets.UTF_8).length;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Test
    public void generateXml() {
        String filePath = "E:\\非结构化数据\\output_hz2.xml";
//        String filePath = "D:\\Work\\CSDL\\非结构化数据\\output.xml";
        long fileSizeLimit = 1_073_741_824 * 100L;
//        long fileSizeLimit = 1_073_741_824;
//        long fileSizeLimit = 1_073;

        try (BufferedWriter writer = Files.newBufferedWriter(Paths.get(filePath))) {
//            writer.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<root>\n");
            File file = new File(filePath);
            long currentSize = file.length();

            while (currentSize < fileSizeLimit) {
                // 将对象转换为JSON字符串
                String xmlData = generateXmlData();
//                String xmlData = "<a><b>"+generateRandomString(4)+"</b><c>"+generateRandomString(6)+"</c></a>\n";

                // 写入文件之前可以检查是否这次写入会导致文件超过限制
                if (currentSize + xmlData.getBytes(StandardCharsets.UTF_8).length > fileSizeLimit) {
                    System.out.println("Reached file size limit. Stopping.");
//                    writer.write("</root>\n");
                    break;
                }

                // 写入JSON数据到文件
                writer.write(xmlData);
//                writer.newLine(); // 可选，根据需要添加换行符以便区分不同的JSON对象

                currentSize += xmlData.getBytes(StandardCharsets.UTF_8).length;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Test
    public void generateText() {
//        String filePath = "E:\\非结构化数据\\output.txt";
        String filePath = "D:\\Work\\CSDL\\非结构化数据\\output.txt";
//        long fileSizeLimit = 1_073_741_824 * 100L;
        long fileSizeLimit = 1_073_741_824;

        try (BufferedWriter writer = Files.newBufferedWriter(Paths.get(filePath))) {
            writer.write("name;id\n");
            File file = new File(filePath);
            long currentSize = file.length();

            while (currentSize < fileSizeLimit) {
                // 将对象转换为JSON字符串
                String textData = generateTextData();

                // 写入文件之前可以检查是否这次写入会导致文件超过限制
                if (currentSize + textData.getBytes(StandardCharsets.UTF_8).length > fileSizeLimit) {
                    System.out.println("Reached file size limit. Stopping.");
                    break;
                }

                // 写入JSON数据到文件
                writer.write(textData);
//                writer.newLine(); // 可选，根据需要添加换行符以便区分不同的JSON对象

                currentSize += textData.getBytes(StandardCharsets.UTF_8).length;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public String generateJsonData() {
        JSONObject jsonObject = new JSONObject();

        jsonObject.put("name", generateRandomString(6));
        jsonObject.put("id", generateRandomInteger(0, 10000));

        String stringJson = jsonObject.toString() + ",";
        return stringJson;
    }

    public String generateXmlData() {
        String xmlData = "";
        xmlData += "<item>";
        xmlData += "<name>" + generateRandomString(6) + "</name>";
        xmlData += "<id>" + generateRandomInteger(0, 10000) + "</id>";
        xmlData += "<card>" + generateRandomInteger(0, 1000) + "</card>";
        xmlData += "<desc>" + generateRandomString(4) + "</desc>";
        xmlData += "</item>\n";
        return xmlData;
    }

    public String generateTextData() {
        String textData = generateRandomString(6) + ";" + generateRandomInteger(0, 10000) + "\n";
        return textData;
    }

    public static String generateRandomString(int length) {
        Random random = new Random();
        StringBuilder sb = new StringBuilder(length);
        for (int i = 0; i < length; i++) {
            // 可以根据需要调整字符集
            int randomChar = random.nextInt(26) + 'a';
            sb.append((char) randomChar);
        }
        return sb.toString();
    }

    public static int generateRandomInteger(int max, int min) {
        int randomNum = (int) (Math.random() * (max - min + 1) + min);
        return randomNum;
    }
}